@@ -1,14 +1,13 @@
 from __future__ import annotations
 import numpy as np
+from numpy.typing import ArrayLike, DTypeLike, _ShapeLike
 import os
 import re
 import glob
 import itertools
-import inspect
 from functools import wraps
-from dask import array as da
 
-from .utils.io import *
+from .utils.io import open_img, open_as_dask, get_scale_from_meta, open_mrc, open_tif
 from .utils import gauss
 from .utils.slicer import *
 from .utils.utilcls import Progress
@@ -54,7 +53,12 @@ def write_docs(func):
     return func
 
 @write_docs
-def array(arr, dtype = None, *, name: str = None, axes: str = None, copy: bool = True) -> ImgArray:
+def array(arr: ArrayLike,
+          dtype: DTypeLike = None, 
+          *,
+          name: str = None,
+          axes: str = None,
+          copy: bool = True) -> ImgArray:
     """
     make an ImgArray object, like ``np.array(x)``
     
@@ -90,7 +94,11 @@ def array(arr, dtype = None, *, name: str = None, axes: str = None, copy: bool =
     return self
 
 @write_docs
-def asarray(arr, dtype = None, *, name: str=None, axes: str=None) -> ImgArray:
+def asarray(arr: ArrayLike,
+            dtype: DTypeLike = None,
+            *, 
+            name: str = None,
+            axes: str = None) -> ImgArray:
     """
     make an ImgArray object, like ``np.asarray(x)``
     
@@ -110,7 +118,12 @@ def asarray(arr, dtype = None, *, name: str=None, axes: str=None) -> ImgArray:
     return array(arr, dtype=dtype, name=name, axes=axes, copy=False)
 
 @write_docs
-def aslazy(arr, dtype=None, *, name:str=None, axes:str=None, chunks="auto") -> LazyImgArray:
+def aslazy(arr: ArrayLike, 
+           dtype: DTypeLike = None,
+           *, 
+           name: str = None,
+           axes: str = None,
+           chunks="auto") -> LazyImgArray:
     """
     Make an LazyImgArray object from other types of array.
     
@@ -127,6 +140,7 @@ def aslazy(arr, dtype=None, *, name:str=None, axes:str=None, chunks="auto") -> L
     LazyImgArray
     
     """
+    from dask import array as da
     if isinstance(arr, MetaArray):
         arr = da.from_array(arr.value, chunks=chunks)
     if isinstance(arr, (np.ndarray, np.memmap)):
@@ -150,40 +164,48 @@ def aslazy(arr, dtype=None, *, name:str=None, axes:str=None, chunks="auto") -> L
     
     return self
 
-@write_docs
-def _template(shape, dtype=np.uint16, *, name:str=None, axes:str=None) -> ImgArray:
-    r"""
-    Make an ImgArray object, like ``np.{npfuncname}``.
 
-    Parameters
-    ----------
-    shape : int or tuple of int
-        Shape of image.
-    {}
-    """    
-
-def np_dispatcher(func):
-    npfunc = getattr(np, func.__name__)
-    @wraps(_template)
-    def _func(*args, name=None, axes=None, **kwargs):
+def inject_numpy_function(func: Callable):
+    npfunc: Callable = getattr(np, func.__name__)
+    @wraps(func)
+    def _func(*args, **kwargs):
+        axes = kwargs.pop("axes", None)
+        name = kwargs.pop("name", None)
         return asarray(npfunc(*args, **kwargs), name=name, axes=axes)
-    _func.__name__ = npfunc.__name__
-    _func.__doc__ = re.sub(r"{npfuncname}", npfunc.__name__, _func.__doc__)
+    _func.__doc__ = (
+        f"""
+        impy version of numpy.{func.__name__}. This function has additional parameters ``axes``
+        and ``name``. Original docstring follows.
+        
+        Additional Parameters
+        ---------------------
+        axes : str, optional
+            Image axes. Must be same length as image dimension.
+        name: str, optional
+            Image name.
+        
+        {npfunc.__doc__}
+        """
+        )
+    _func.__annotations__.update({"return": ImgArray})
     return _func
 
-@np_dispatcher
-def zeros(): ...
+@inject_numpy_function
+def zeros(shape: _ShapeLike, dtype: DTypeLike = np.uint16, *, name: str = None, axes: str = None): ...
     
-@np_dispatcher
-def empty(): ...
+@inject_numpy_function
+def empty(shape: _ShapeLike, dtype: DTypeLike = np.uint16, *, name: str = None, axes: str = None): ...
 
-@np_dispatcher
-def ones(): ...
+@inject_numpy_function
+def ones(shape: _ShapeLike, dtype: DTypeLike = np.uint16, *, name: str = None, axes: str = None): ...
 
-@np_dispatcher
-def full(): ...
+@inject_numpy_function
+def full(shape: _ShapeLike, fill_value: Any, dtype: DTypeLike = np.uint16, *, name: str = None, axes: str = None): ...
 
-def gaussian_kernel(shape:tuple[int, ...], sigma=1.0, peak:float=1.0) -> ImgArray:
+
+def gaussian_kernel(shape: _ShapeLike, 
+                    sigma: nDFloat = 1.0,
+                    peak: float = 1.0) -> ImgArray:
     """
     Make an Gaussian kernel or Gaussian image.
 
@@ -211,7 +233,9 @@ def gaussian_kernel(shape:tuple[int, ...], sigma=1.0, peak:float=1.0) -> ImgArra
     return ker
 
 
-def circular_mask(radius:nDFloat, shape:tuple[int,...], center:str|tuple[float,...]="center") -> ImgArray:
+def circular_mask(radius: nDFloat, 
+                  shape: _ShapeLike,
+                  center: str | tuple[float, ...] = "center") -> ImgArray:
     """
     Make a circular or ellipsoid shaped mask. Region close to center will be filled with ``False``. 
 
@@ -246,7 +270,7 @@ def circular_mask(radius:nDFloat, shape:tuple[int,...], center:str|tuple[float,.
     return array(s > 1.0, dtype=bool, axes=axes)
 
 
-def sample_image(name:str) -> ImgArray:
+def sample_image(name: str) -> ImgArray:
     """
     Get sample images from ``skimage`` and convert it into ImgArray.
 
@@ -272,8 +296,12 @@ def sample_image(name:str) -> ImgArray:
 #   Imread functions
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
 
-def imread(path:str, dtype:str=None, key:str=None, *, squeeze:bool=False) -> ImgArray:
-    """
+def imread(path: str,
+           dtype: DTypeLike = None,
+           key: str = None,
+           *, 
+           squeeze: bool = False) -> ImgArray:
+    r"""
     Load image(s) from a path. You can read list of images from directories with wildcards or ``"$"``
     in ``path``.
 
@@ -341,7 +369,7 @@ def imread(path:str, dtype:str=None, key:str=None, *, squeeze:bool=False) -> Img
         history = []
         
     if is_memmap:
-        sl = axis_targeted_slicing(img, axes, key)
+        sl = axis_targeted_slicing(img.ndim, axes, key)
         axes = "".join(a for a, k in zip(axes, sl) if not isinstance(k, int))
         img = np.asarray(img[sl], dtype=dtype)
     
@@ -375,7 +403,7 @@ def imread(path:str, dtype:str=None, key:str=None, *, squeeze:bool=False) -> Img
         
         return self.sort_axes().as_img_type(dtype) # arrange in tzcyx-order
 
-def _imread_glob(path:str, squeeze:bool=False, **kwargs) -> ImgArray:
+def _imread_glob(path: str, squeeze: bool = False, **kwargs) -> ImgArray:
     """
     Read images recursively from a directory, and stack them into one ImgArray.
 
@@ -409,7 +437,10 @@ def _imread_glob(path:str, squeeze:bool=False, **kwargs) -> ImgArray:
     out.temp = paths
     return out
 
-def _imread_stack(path:str, dtype=None, key:str=None, squeeze=False):
+def _imread_stack(path: str, 
+                  dtype: DTypeLike = None,
+                  key: str = None,
+                  squeeze: bool = False):
     r"""
     Read separate image files using formated string. This function is useful when files/folders
     are named in a certain rule, such as ".../pos_0/img_0.tif", ".../pos_0/img_1.tif".
@@ -529,7 +560,9 @@ def _imread_stack(path:str, dtype=None, key:str=None, squeeze=False):
         self = np.squeeze(self)
     return self.sort_axes()
 
-def imread_collection(path:str|list[str], filt=None) -> DataList:
+
+def imread_collection(path: str | list[str], 
+                      filt: Callable[[np.ndarray], bool] = None) -> DataList:
     """
     Open images as ImgArray and store them in DataList.
 
@@ -565,7 +598,11 @@ def imread_collection(path:str|list[str], filt=None) -> DataList:
             arrlist.append(img)
     return arrlist
 
-def lazy_imread(path, chunks="default", *, squeeze:bool=False) -> LazyImgArray:
+
+def lazy_imread(path: str, 
+                chunks="default",
+                *, 
+                squeeze: bool = False) -> LazyImgArray:
     """
     Read an image lazily. Image file is first opened as an memory map, and subsequently converted
     to `numpy.ndarray` or `cupy.ndarray` chunkwise by `dask.array.map_blocks`.
@@ -603,7 +640,7 @@ def lazy_imread(path, chunks="default", *, squeeze:bool=False) -> LazyImgArray:
         
     if squeeze:
         axes = "".join(a for i, a in enumerate(axes) if img.shape[i] > 1)
-        img = da.squeeze(img)
+        img = np.squeeze(img)
         
     self = LazyImgArray(img, name=name, axes=axes, dirpath=dirpath, 
                         history=history, metadata=metadata)
@@ -616,6 +653,7 @@ def lazy_imread(path, chunks="default", *, squeeze:bool=False) -> LazyImgArray:
         self.set_scale(**scale)
         return self.sort_axes()
 
+
 def _lazy_imread_glob(path:str, squeeze=False, **kwargs) -> LazyImgArray:
     """
     Read images recursively from a directory, and stack them into one LazyImgArray.
@@ -629,7 +667,7 @@ def _lazy_imread_glob(path:str, squeeze=False, **kwargs) -> LazyImgArray:
     path = str(path)
     paths = glob.glob(path, recursive=True)
         
-    imgs = []
+    imgs: list[LazyImgArray] = []
     for path in paths:
         imgl = lazy_imread(path, **kwargs)
         imgs.append(imgl)
@@ -637,13 +675,14 @@ def _lazy_imread_glob(path:str, squeeze=False, **kwargs) -> LazyImgArray:
     if len(imgs) == 0:
         raise RuntimeError("Could not read any images.")
     
-    out = da.stack([i.img for i in imgs], axis=0)
+    from dask import array as da
+    out = da.stack([i.value for i in imgs], axis=0)
     out = LazyImgArray(out)
     out._set_info(imgs[0], new_axes="p"+str(imgs[0].axes))
     
     if squeeze:
         axes = "".join(a for i, a in enumerate(out.axes) if out.shape[i] > 1)
-        img = da.squeeze(out.img)
+        img = da.squeeze(out.value)
         out = LazyImgArray(img)
         out._set_info(imgs[0], new_axes=axes)
     try:
@@ -654,7 +693,8 @@ def _lazy_imread_glob(path:str, squeeze=False, **kwargs) -> LazyImgArray:
     
     return out
 
-def read_meta(path:str) -> dict[str]:
+
+def read_meta(path: str) -> dict[str]:
     """
     Read the metadata of an image file. 
 
@@ -673,13 +713,14 @@ def read_meta(path:str) -> dict[str]:
         - "tags": tiff tags
         
     """    
-    fname, fext = os.path.splitext(os.path.basename(path))
-    
+    path = str(path)
+    _, fext = os.path.splitext(path)
+    meta, _ = open_as_dask(path, chunks="default")
     if fext in (".tif", ".tiff"):
         meta = open_tif(path)
-    elif fext in (".mrc", ".rec"):
+    elif fext in (".mrc", ".rec", ".map"):
         meta = open_mrc(path)
     else:
         raise ValueError("Unsupported file extension.")
     
-    return meta
\ No newline at end of file
+    return meta
